!==========================================
! Generate symmetry relations between different k-points
!==========================================
SUBROUTINE generate_symrel( symm, latt, dx, dy, dz, NSUB, FOUM, DEG, NFOUM, NDEG)

  USE nrtype
  USE nrutil
  USE data
  IMPLICIT NONE
        
  INTEGER, INTENT(IN) :: NSUB
  INTEGER, INTENT(OUT) :: FOUM(NSUB,NSUB), DEG(NSUB)
  INTEGER, INTENT(OUT) :: NFOUM(NSUB,NSUB), NDEG(NSUB)
  LOGICAL :: equiva, lgamma
  LOGICAL, ALLOCATABLE :: mask(:)
  INTEGER ::  npoints, i, j, isym, qa, qb, qc,pp,ss,rr
  REAL(DP) :: bg(3,3), at(3,3), x(3), p,s,r,xm(3), xp(3),OPER(3,3)
  REAL(DP) :: dx, dy, dz
  REAL(DP), PARAMETER :: eps = 1.e-6
  REAL(DP), ALLOCATABLE :: peso(:), kt(:,:), is(:,:,:)


  TYPE(symmetry) :: symm
  TYPE(lattice) :: latt
  
  qa = latt%qa
  qb = latt%qb
  qc = latt%qc
  bg = latt%bg
  lgamma = latt%lgamma
  
  DO i = 1,NSUB
   DO j= 1,NSUB
        FOUM(i,j) = 0
        NFOUM(i,j) = 0
   ENDDO
        DEG(i) = 0
        NDEG(i) = 0
   ENDDO
  
!---------------------------------------------------------------------------------
! If you don't want to go to the edge of the BZ set dx, and/or dy and/or dz < 1.0
! Defaults are dx=dy=dz=1.0 (normal behaviour)
!---------------------------------------------------------------------------------

  print'(/''Generating IBZ points ....'',3i5)',qa,qb,qc

  npoints = qa * qb * qc
  allocate( mask(npoints), peso(npoints), kt(3,npoints), is(3,3,symm%nsym) )
  kt = 0
!----------------------------
! generate points
!----------------------------
  i = 0
  lgamma = .TRUE.
  if(.not.lgamma)then                           ! reticolo generale
     do rr = 1, qa
        do pp = 1, qb
           do ss = 1, qc
            r =REAL(rr)
            s = REAL(ss)
            p = REAL(pp)
              i = i + 1
              kt(1,i) = &
                   ( 2*r - qa - 1 ) /2 / qa * bg(1,1) + &
                   ( 2*p - qa - 1 ) /2 / qa * bg(1,2) + &
                   ( 2*s - qa - 1 ) /2 / qa * bg(1,3)
              if( dx < 0 ) then
                 kt(1,i) = (0.5 - kt(1,i) * dx)
              else
                 kt(1,i) = kt(1,i) * dx
              endif
              kt(2,i) = &
                   ( 2*r - qb - 1 ) /2 / qb * bg(2,1) + &
                   ( 2*p - qb - 1 ) /2 / qb * bg(2,2) + &
                   ( 2*s - qb - 1 ) /2 / qb * bg(2,3)
              if( dy < 0 ) then
                 kt(2,i) = (0.5 - kt(2,i) * dy)
              else
                 kt(2,i) = kt(2,i) * dy
              endif
              kt(3,i) = &
                   ( 2*r - qc - 1 ) /2 / qc * bg(3,1) + &
                   ( 2*p - qc - 1 ) /2 / qc * bg(3,2) + &
                   ( 2*s - qc - 1 ) /2 / qc * bg(3,3)
              if( dz < 0 ) then
                 kt(3,i) = (0.5 - kt(3,i) * dz)
              else
                 kt(3,i) = kt(3,i) * dz
              endif
           enddo
        enddo
     enddo
  else
     do rr = 1, qa
        do pp = 1, qb
           do ss = 1, qc
              i = i + 1
                     r =REAL(rr)
                     s = REAL(ss)
                     p = REAL(pp)
              kt(1,i) = &
                   ( r - 1 ) / qa * bg(1,1) + &
                   ( p - 1 ) / qa * bg(1,2) + &
                   ( s - 1 ) / qa * bg(1,3)
              kt(2,i) = &
                   ( r - 1 ) / qb * bg(2,1) + &
                   ( p - 1 ) / qb * bg(2,2) + &
                   ( s - 1 ) / qb * bg(2,3)
              kt(3,i) = &
                   ( r - 1 ) / qc * bg(3,1) + &
                   ( p - 1 ) / qc * bg(3,2) + &
                   ( s - 1 ) / qc * bg(3,3)
           enddo
        enddo
     enddo
  endif

  call inve( bg, at )
  at = transpose(at)

  kt = matmul( transpose(at), kt )
  kt = kt - aint(2*kt)

  if( .not. symm%lsymm ) then
     mask=.true.
     peso = 1
     goto 10
  endif


  do i=1,symm%nsym
     is(:,:,i) = matmul( transpose(bg), matmul(symm%iscryst(:,:,i), at ) )
     is(:,:,i) = transpose( is(:,:,i) )
  enddo
  
  
 !----------------------------
 ! now reduce them to the IBZ
 !----------------------------
  mask = .true.
  peso = 1
  do i = 1, npoints
        do j = 1, npoints
              dosym: do isym = 1, symm%nsym
                 x(1) = is(1,1,isym) * kt(1,j) + is(1,2,isym) * kt(2,j) + is(1,3,isym) * kt(3,j) 
                 x(2) = is(2,1,isym) * kt(1,j) + is(2,2,isym) * kt(2,j) + is(2,3,isym) * kt(3,j) 
                 x(3) = is(3,1,isym) * kt(1,j) + is(3,2,isym) * kt(2,j) + is(3,3,isym) * kt(3,j) 
                 equiva = &
                      ( abs( x(1) - kt(1,i)  - nint( x(1) - kt(1,i) ) ) < eps .and. &
                      abs( x(2) - kt(2,i)  - nint( x(2) - kt(2,i) ) ) < eps .and. &
                      abs( x(3) - kt(3,i)  - nint( x(3) - kt(3,i) ) ) < eps )
  !------------------------------------------------------------------------------------
  ! keep only K, eliminate -K (the dynamical matrix has equal eigenvalues at K and -K)
  !------------------------------------------------------------------------------------
                 if( .not. equiva ) THEN 
                      equiva = &
                      ( abs( x(1) + kt(1,i)  - nint( x(1) + kt(1,i) ) ) < eps .and. &
                      abs( x(2) + kt(2,i)  - nint( x(2) + kt(2,i) ) ) < eps .and. &
                      abs( x(3) + kt(3,i)  - nint( x(3) + kt(3,i) ) ) < eps )
                      IF (equiva) THEN
                              NDEG(i) = NDEG(i)+1
                              NFOUM(i,NDEG(i)) = j
                      ENDIF
                 ENDIF
                 if(equiva)then
                    mask(j) = .false.
                    peso(i) =  peso(i) + 1
                    DEG(i) = DEG(i)+1
                    FOUM(i,DEG(i)) = j
                    exit dosym
                 endif
              enddo dosym
        enddo
  enddo
  
10 continue
  
END SUBROUTINE generate_symrel


